import { ElementAnimateConfig } from 'zrender';
import { TextAlign } from 'zrender/lib/core/types';
import { getOpacityColor } from './helper';

export const LIGAND_WIDTH = 120;
export const LIGAND_WIDTH_HALF = LIGAND_WIDTH / 2;
export const SCALE_1 = 1;
export const SCALE_X = 1.5;
export const NORMAL_OPACITY = 1;
export const FADEOUT_OPACITY = 0.1;
export let ELEMENT_Z = 1;
export const ELEMENT_Z2 = 1;
export const addElementZ = () => ++ELEMENT_Z;

const LIGAND_RADIUS = 2;
const LIGAND_IMAGE_WIDTH = LIGAND_WIDTH - LIGAND_RADIUS;
const LIGAND_IMAGE_OFFSET = LIGAND_RADIUS / 2;
const LIGAND_DEFAULT_COLOR = '#F0F1F6';
const LIGAND_SELECTED_COLOR = '#3E4BD5';
const LIGAND_HOVER_COLOR = '#93EBED';
const LIGAND_FIRSTADD_COLOR = '#C6A3FF';
const LIGAND_EXP_COLOR = '#a63cf9';
const LIGAND_CALC_COLOR = '#4b95ff';
const LIGAND_DEFAULT_SHADOWBLUR = 0;
const LIGAND_HIGHLIGHT_SHADOWBLUR = 14;
const LIGAND_STROKE_WIDTH = 1;
const LIGAND_INITIAL_FILL = 'rgba(0,0,0,0)';
const LIGAND_BGRECT_FILL = '#f8f8fc';
const LIGAND_ORDER_ORIGIN = 45;
const LIGAND_ORDER_OFFSET = 5;
const LIGAND_ORDER_FILL = '#fff';
const LIGAND_ORDER_FONTWEIGHT = 600;
const LIGAND_ORDER_ALIGN: TextAlign = 'center';
const LIGAND_ORDER_WIDTH = 14;
const LIGAND_ORDER_BORDERRADIUS = LIGAND_ORDER_WIDTH / 2;
const LIGAND_TEXT_FONTSIZE = 12;
const LIGAND_TEXT_LINEHEIGHT = 24;
const LIGAND_TEXT_DEFAULT_FILL = '#999';
const LIGAND_TEXT_BACKGROUND_OPACITY = 0.3;

const EDGE_DEFAULT_COLOR = '#C3C6CB';
const EDGE_HOVER_COLOR = '#616266';
const EDGE_VIRTUAL_COLOR = '#E07575';
const EDGE_STROKE_WIDTH = 1.5;
const EDGE_VISUAL_LINEDASH = [2, 6];
const EDGE_LINEDASH = null;
const EDGE_TEXT_ALIGIN: TextAlign = 'center';
const EDGE_ARROW_WIDTH = 7;

const ANIMATE_DURATION = 200;
const ANIMATE_EASING = 'cubicOut';

export const LIGAND_EXP_CALC_DIS = 6;
export const LIGAND_EXP_CALC_LINEHEIGHT = 26;

export const ANIMATE_CONFIG: ElementAnimateConfig = {
    duration: ANIMATE_DURATION,
    easing: ANIMATE_EASING,
};

export const LIGAND_RECT_STYLE = {
    SELECTED: {
        STROKE: LIGAND_SELECTED_COLOR,
    },
    HOVER: {
        STROKE: LIGAND_HOVER_COLOR,
    },
    RELATEDHOVER: {
        STROKE: LIGAND_DEFAULT_COLOR,
    },
    FIRSTADD: {
        STROKE: LIGAND_FIRSTADD_COLOR,
    },
    DEFAULT: {
        STROKE: LIGAND_DEFAULT_COLOR,
    },
    INITIAL: {
        FILL: LIGAND_INITIAL_FILL,
        RADIUS: LIGAND_RADIUS,
        WIDTH: LIGAND_WIDTH,
        HEIGHT: LIGAND_WIDTH,
        ZLEVEL_THIRD: ELEMENT_Z2,
        LINEWIDTH: LIGAND_STROKE_WIDTH,
    },
};

export const LIGAND_IMAGE_STYLE = {
    POSITION: {
        OFFSET: LIGAND_IMAGE_OFFSET,
    },
    INITIAL: {
        WIDTH: LIGAND_IMAGE_WIDTH,
        HEIGHT: LIGAND_IMAGE_WIDTH,
        ZLEVEL_THIRD: ELEMENT_Z2 + 1,
    },
};

export const LIGAND_BGRECT_STYLE = {
    SELECTED: {
        SHADOWCOLOR: LIGAND_SELECTED_COLOR,
        SHADOWBLUR: LIGAND_HIGHLIGHT_SHADOWBLUR,
    },
    HOVER: {
        SHADOWCOLOR: LIGAND_HOVER_COLOR,
        SHADOWBLUR: LIGAND_HIGHLIGHT_SHADOWBLUR,
    },
    RELATEDHOVER: {
        SHADOWBLUR: LIGAND_DEFAULT_SHADOWBLUR,
    },
    FIRSTADD: {
        SHADOWCOLOR: LIGAND_FIRSTADD_COLOR,
        SHADOWBLUR: LIGAND_HIGHLIGHT_SHADOWBLUR,
    },
    DEFAULT: {
        SHADOWBLUR: LIGAND_DEFAULT_SHADOWBLUR,
    },
    INITIAL: {
        FILL: LIGAND_BGRECT_FILL,
        RADIUS: LIGAND_RADIUS,
        WIDTH: LIGAND_WIDTH,
        HEIGHT: LIGAND_WIDTH,
        ZLEVEL_THIRD: ELEMENT_Z2,
        LINEWIDTH: LIGAND_STROKE_WIDTH,
    },
};

export const LIGAND_ORDER_STYLE = {
    POSITION: {
        ORIGIN: LIGAND_ORDER_ORIGIN,
        OFFSET: LIGAND_ORDER_OFFSET,
    },
    INITIAL: {
        FILL: LIGAND_ORDER_FILL,
        FONTWEIGHT: LIGAND_ORDER_FONTWEIGHT,
        ALIGN: LIGAND_ORDER_ALIGN,
        BACKGROUNDCOLOR: LIGAND_SELECTED_COLOR,
        WIDTH: LIGAND_ORDER_WIDTH,
        HEIGHT: LIGAND_ORDER_WIDTH,
        BORDERRADIUS: LIGAND_ORDER_BORDERRADIUS,
        ZLEVEL_THIRD: ELEMENT_Z2 + 2,
    },
};

export const LIGAND_TEXT_STYLE = {
    SELECTED: {
        BACKGROUNDCOLOR: getOpacityColor(LIGAND_SELECTED_COLOR, LIGAND_TEXT_BACKGROUND_OPACITY),
        FILL: LIGAND_SELECTED_COLOR,
    },
    HOVER: {
        BACKGROUNDCOLOR: getOpacityColor(LIGAND_HOVER_COLOR, LIGAND_TEXT_BACKGROUND_OPACITY),
        FILL: LIGAND_HOVER_COLOR,
    },
    RELATEDHOVER: {
        BACKGROUNDCOLOR: getOpacityColor(EDGE_DEFAULT_COLOR, LIGAND_TEXT_BACKGROUND_OPACITY),
        FILL: LIGAND_TEXT_DEFAULT_FILL,
    },
    FIRSTADD: {
        BACKGROUNDCOLOR: getOpacityColor(LIGAND_FIRSTADD_COLOR, LIGAND_TEXT_BACKGROUND_OPACITY),
        FILL: LIGAND_FIRSTADD_COLOR,
    },
    DEFAULT: {
        BACKGROUNDCOLOR: getOpacityColor(EDGE_DEFAULT_COLOR, LIGAND_TEXT_BACKGROUND_OPACITY),
        FILL: LIGAND_TEXT_DEFAULT_FILL,
    },
    INITIAL: {
        FONT_SIZE: LIGAND_TEXT_FONTSIZE,
        LINE_HEIGHT: LIGAND_TEXT_LINEHEIGHT,
        ALIGN: EDGE_TEXT_ALIGIN,
        ZLEVEL_THIRD: ELEMENT_Z2 + 2,
    },
};

export const LIGAND_EXP_TEXT_STYLE = {
    DEFAULT: {
        FILL: LIGAND_EXP_COLOR,
    },
};

export const LIGAND_CALC_TEXT_STYLE = {
    DEFAULT: {
        FILL: LIGAND_CALC_COLOR,
    },
};

export const EDGE_LINE_STYLE = {
    VIRTUAL: {
        STROKE: EDGE_VIRTUAL_COLOR,
    },
    SELECTED: {
        STROKE: LIGAND_SELECTED_COLOR,
    },
    HOVER: {
        STROKE: EDGE_HOVER_COLOR,
    },
    RELATEDHOVER: {
        STROKE: EDGE_HOVER_COLOR,
    },
    DEFAULT: {
        STROKE: EDGE_DEFAULT_COLOR,
    },
    INITIAL: {
        LINEWIDTH: EDGE_STROKE_WIDTH,
        LINEDASH: EDGE_LINEDASH,
        VISUAL_LINEDASH: EDGE_VISUAL_LINEDASH,
    },
};

export const EDGE_TEXT_STYLE = {
    VIRTUAL: {
        Fill: EDGE_VIRTUAL_COLOR,
    },
    SELECTED: {
        Fill: LIGAND_SELECTED_COLOR,
    },
    HOVER: {
        Fill: EDGE_HOVER_COLOR,
    },
    RELATEDHOVER: {
        Fill: EDGE_HOVER_COLOR,
    },
    DEFAULT: {
        Fill: EDGE_DEFAULT_COLOR,
    },
    INITIAL: {
        ALIGN: EDGE_TEXT_ALIGIN,
    },
};

export const EDGE_ARROW_STYLE = {
    VIRTUAL: {
        FILL: EDGE_VIRTUAL_COLOR,
    },
    SELECTED: {
        FILL: LIGAND_SELECTED_COLOR,
    },
    HOVER: {
        FILL: EDGE_HOVER_COLOR,
    },
    RELATEDHOVER: {
        FILL: EDGE_HOVER_COLOR,
    },
    DEFAULT: {
        FILL: EDGE_DEFAULT_COLOR,
    },
    INITIAL: {
        WIDTH: EDGE_ARROW_WIDTH,
        HEIGHT: EDGE_ARROW_WIDTH,
    },
};
